/*
 * Copyright 2010, cluster:Systems CSG GmbH
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package net.sourceforge.mailprobe.check;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.InetAddress;
import java.net.Socket;
import java.util.Date;
import java.util.Map;
import java.util.TreeMap;
import javax.naming.NamingException;
import java.util.logging.Level;
import java.util.logging.Logger;
import net.sourceforge.mailprobe.smtp.MXLookup;
import net.sourceforge.mailprobe.util.ExecStats;

/**
 * Class for pinging SMTP-Servers.
 * 
 * @author Thomas Tesche <thomas.tesche@clustersystems.de>
 */
public class SMTPServerCheck {

  final private static Logger logger = Logger.getLogger(SMTPServerCheck.class.getName());

  private int hear(BufferedReader in) throws IOException {
    String line = null;
    int res = 0;

    while ((line = in.readLine()) != null) {
      String pfx = line.substring(0, 3);
      try {
        res = Integer.parseInt(pfx);
      } catch (Exception ex) {
        res = -1;
      }
      if (line.charAt(3) != '-') {
        break;
      }
    }

    return res;
  }

  private void say(BufferedWriter wr, String text)
          throws IOException {
    wr.write(text + "\r\n");
    wr.flush();

    return;
  }

  /**
   * Exceute a ping on one or more mail server listed in the MX record of mailDomainName.
   * The first successful ping stops further checking of the remaining servers.
   *
   * @param mailDomainName the mail domain name to find and ping mails servers
   * @param localhostName the name of from which the test is performed. Used in EHLO.
   * @return true in case at least one mail server responds with a valid dialog.
   */
  public ExecStats simpleMXPing(String mailDomainName, String localhostName) {

    TreeMap<Integer, String> mxList = null;
    // success per default = false, startDate = new Date()
    ExecStats result = new ExecStats(new Date());

    try {
      // If there couldn't be found the mailDomainName via DNS an exception is thrown
      mxList = new MXLookup().getMXHosts(mailDomainName);

      for (Map.Entry<Integer, String> entry : mxList.entrySet()) {
        // Now, do the SMTP validation, try each mail exchanger
        if (simplePing((String) mxList.get(entry.getKey()), localhostName).isSuccess()) {
          result.setSuccess(true);
          break;
        }
      }

    } catch (NamingException ex) {
      // The naming exception will be thrown in case of zero f
      logger.log(Level.FINE, "mailDomainHost not found via DNS.", ex);
    }

    result.setEndDate(new Date());
    return result;
  }

  /**
   * Ping a certain mail exchange server (SMTP)
   *
   * @param exchangerHostName name of the mail exchanger
   * @param localhostName name used for EHLO. Could be null. IP is also possible [xxx.xxx.xxx.xxx]
   * @return true if the connect was successful and the answers
   *         from the exchanger are correct. False otherwise.
   */
  public ExecStats simplePing(String exchangerHostName, String localhostName) {

    // success per default = false, startDate = new Date()
    ExecStats result = new ExecStats(new Date());

    try {

      if (localhostName == null) {
        localhostName = InetAddress.getLocalHost().getHostName();
      }
      int res;
      Socket socket = new Socket(exchangerHostName, 25);
      BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(socket.getInputStream()));
      BufferedWriter bufferedWriter = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream()));
      res = hear(bufferedReader);
      if (res != 220) {
        throw new Exception("Invalid header of SMTP server.");
      }
      // Some mail server checks the requesting IP against the IP from the DNS for the hostName
      // and rejects the communication of they don't match.
      // In such case use the gateway hostname or IP-address.
      // IP in brackets [xxx.xxx.xxx.xx]. See http://cr.yp.to/smtp/helo.html
      say(bufferedWriter, "EHLO " + localhostName);
      res = hear(bufferedReader);
      if (res != 250) {
        throw new Exception("Not ESMTP");
      }
      say(bufferedWriter, "NOOP");
      res = hear(bufferedReader);
      if (res != 250) {
        throw new Exception("NOOP is not supported");
      }
      say(bufferedWriter, "QUIT");
      res = hear(bufferedReader);
      if (res != 221) {
        throw new Exception("QUIT doesn't work as expected");
      }
      result.setSuccess(true);

      bufferedReader.close();
      bufferedWriter.close();
      socket.close();
    } catch (Exception ex) {
      // Try next host
      logger.log(Level.FINE, "Error while testing certain mail hosts: " + exchangerHostName, ex);
    }

    result.setEndDate(new Date());
    return result;
  }
}
