/*
 * Copyright 2010, cluster:Systems CSG GmbH
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package net.sourceforge.mailprobe.check;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.Date;
import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.mail.Flags;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import net.sourceforge.mailprobe.smtp.SMTPSender;
import net.sourceforge.mailprobe.store.Mailstore;
import net.sourceforge.mailprobe.util.ExecStats;
import net.sourceforge.mailprobe.util.Protocol;

/**
 * SendReceiveCheck for the flow smtp-imap/pop3.
 * The sended message is composed either in this class or outside of this class.
 *
 * @author Thomas Tesche <thomas.tesche@clustersystems.de>
 */
public class SendReceiveCheck {

  private static final Logger logger = Logger.getLogger(SendReceiveCheck.class.getName());

  /**
   * Takes a MimeMessage, sends the message and retrieves the message from the recipients
   * postbox.
   *
   * @param smtpProtocol
   * @param smtpHostname
   * @param smtpEhloLocalhost
   * @param smtpPort
   * @param smtpUsername
   * @param smtpPassword
   * @param smtpUseStarttls
   * @param mailstoreProtocol
   * @param mailstoreHostname
   * @param mailstorePort
   * @param mailstoreUsername
   * @param mailstorePassword
   * @param mailstoreUseStarttls
   * @param mimeMessage
   * @param timeout
   * @return result of execution in form of an ExecStats object
   */
  public ExecStats check(Protocol smtpProtocol, String smtpHostname, String smtpEhloLocalhost, String smtpPort,
          String smtpUsername, String smtpPassword, boolean smtpUseStarttls,
          Protocol mailstoreProtocol, String mailstoreHostname, String mailstorePort,
          String mailstoreUsername, String mailstorePassword, boolean mailstoreUseStarttls,
          MimeMessage mimeMessage, long timeout) {

    SMTPSender smtpSender = new SMTPSender(smtpProtocol, smtpHostname, smtpEhloLocalhost, smtpPort,
            smtpUsername, smtpPassword, smtpUseStarttls);
    String uuid = UUID.randomUUID().toString();

    ByteArrayOutputStream baos = null;
    try {
      int size = mimeMessage.getSize();
      if (size > 0) {
        baos = new ByteArrayOutputStream(size);
      } else {
        baos = new ByteArrayOutputStream();
      }
      mimeMessage.writeTo(baos);
      baos.close();

    } catch (Exception exception) {
      ExecStats result = new ExecStats(new Date());
      result.setSuccess(false);
      result.setEndDate(new Date());
      logger.log(Level.WARNING, "While copying mimeMessage", exception);
      return result;
    }

    ExecStats result = new ExecStats(new Date());
    try {

      result = smtpSender.sendMessage(new ByteArrayInputStream(baos.toByteArray()), uuid);

      Date timeoutDate = new Date(result.getStartDate().getTime() + timeout);

      Mailstore mailstore = new Mailstore(mailstoreProtocol, mailstoreHostname,
              mailstorePort, mailstoreUsername, mailstorePassword, mailstoreUseStarttls);

      boolean foundMail = false;
      StoreChecker storeChecker = StoreChecker.getInstance();

      while (!foundMail) {
        // Check for 'inner' timeout
        if (timeoutDate.getTime() < (new Date()).getTime()) {
          //logger.log(Level.FINE, "Calculated max end date: " + timeoutDate.getTime() + ", now: " + (new Date()).getTime());
          throw new InterruptedException("Inner timeout in standardCheck occured");
        }
        Thread.sleep(5000);
        logger.log(Level.FINE, "Before storeChecker on UUID: {0}", uuid);
        foundMail = storeChecker.searchUUID(mailstore, uuid);
        logger.log(Level.FINE, "After storeChecker on UUID: {0}", uuid);
      }
    } catch (InterruptedException interruptedException) {
      result.setSuccess(false);
      result.setInnerTimeoutOccured(true);
      logger.log(Level.WARNING, interruptedException.getMessage());

    } catch (Exception ex) {
      result.setSuccess(false);
      logger.log(Level.WARNING, ex.getMessage());
    }

    result.setEndDate(new Date());
    return result;
  }

  /**
   * Composes a MimeMessage, sends the message and retrieves the message from the recipients
   * postbox.
   *
   * @deprecated
   *
   * @param smtpProtocol
   * @param smtpHostname
   * @param smtpEhloLocalhost
   * @param smtpPort
   * @param smtpUsername
   * @param smtpPassword
   * @param smtpUseStarttls
   * @param mailstoreProtocol
   * @param mailstoreHostname
   * @param mailstorePort
   * @param mailstoreUsername
   * @param mailstorePassword
   * @param mailstoreUseStarttls
   * @param mailText
   * @param toAddress
   * @param fromAddress
   * @param timeout
   * @return result of execution in form of an ExecStats object
   */
  public ExecStats standardCheck(Protocol smtpProtocol, String smtpHostname, String smtpEhloLocalhost, String smtpPort,
          String smtpUsername, String smtpPassword, boolean smtpUseStarttls,
          Protocol mailstoreProtocol, String mailstoreHostname, String mailstorePort,
          String mailstoreUsername, String mailstorePassword, boolean mailstoreUseStarttls,
          String mailText, String toAddress, String fromAddress, long timeout) {

    SMTPSender smtpSender = new SMTPSender(smtpProtocol, smtpHostname, smtpEhloLocalhost, smtpPort,
            smtpUsername, smtpPassword, smtpUseStarttls);
    String uuid = UUID.randomUUID().toString();

    ExecStats result = smtpSender.sendStandardMessage(toAddress, fromAddress, uuid, mailText);
    Date timeoutDate = new Date(result.getStartDate().getTime() + timeout);

    Mailstore mailstore = new Mailstore(mailstoreProtocol, mailstoreHostname,
            mailstorePort, mailstoreUsername, mailstorePassword, mailstoreUseStarttls);

    try {
      boolean foundMail = false;
      Message msg = null;
      while (!foundMail) {
        // Check for 'inner' timeout
        if (timeoutDate.getTime() < (new Date()).getTime()) {
          //logger.log(Level.FINE, "Calculated max end date: " + timeoutDate.getTime() + ", now: " + (new Date()).getTime());
          throw new InterruptedException("Inner timeout in standardCheck occured");
        }
        Thread.sleep(5000);
        boolean connectSuccesful = mailstore.connect();
        if (!connectSuccesful) {
          throw new MessagingException("Connect to INBOX wasn't successful.");
        }
        msg = mailstore.searchForSubjectAndUUID("[MAILPROBE]", uuid);

        if (msg != null) {
          foundMail = true;
          msg.setFlag(Flags.Flag.DELETED, true);
          mailstore.disconnect(true);
        } else {
          mailstore.disconnect(false);
        }
      }
    } catch (InterruptedException interruptedException) {
      result.setSuccess(false);
      result.setInnerTimeoutOccured(true);
      logger.log(Level.WARNING, "", interruptedException);

    } catch (MessagingException messagingException) {
      result.setSuccess(false);
      logger.log(Level.WARNING, "", messagingException);
    }

    result.setEndDate(new Date());
    return result;
  }
}
