/*
 * Copyright 2010, cluster:Systems CSG GmbH
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package net.sourceforge.mailprobe.check;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.mail.Flags;
import javax.mail.Message;
import javax.mail.MessagingException;
import net.sourceforge.mailprobe.store.Mailstore;

/**
 * Singleton for searching in Mailstores.
 * See Bloch, Effective Java, 2nd edition, p.17
 *
 * @author Thomas Tesche <thomas.tesche@clustersystems.de>
 */
public class StoreChecker {

  private static final StoreChecker INSTANCE = new StoreChecker();
  private Map<String, StoreUUIDHolder> mailstoreMap = new HashMap<String, StoreUUIDHolder>();
  private Set<String> foundUUIDs = new HashSet<String>();
  /** Time of last check run. */
  private long lastCheckTime = new Date().getTime();
  /** Minimal Time between to two checks. */
  private static final long MIN_CHECK_DELTA = 5000l;
  private static final Logger logger = Logger.getLogger(StoreChecker.class.getName());

  private StoreChecker() {
  }

  public static StoreChecker getInstance() {
    return INSTANCE;
  }

  /**
   * Search for the email with the UUID in the given mailstore. If the email
   * was found the UUID is marked as found and deleted from the mailstore.
   *
   * @param mailstore to search the email in
   * @param UUID
   * @return true if the email with the UUID was found in the mailstore
   */
  public synchronized boolean searchUUID(Mailstore mailstore, String UUID) throws MessagingException {

    if (foundUUIDs.remove(UUID)) {
      // email was found before and is already removed from the store
      logger.log(Level.FINE, "Found UUID whithout search UUID: {0}", UUID);
      return true;
    } else {
      if (mailstoreMap.containsKey(mailstore.toString())) {
        // The current mailstore to check is already initialised in our map
        mailstoreMap.get(mailstore.toString()).addUUID(UUID);
      } else {
        StoreUUIDHolder storeUUIDHolder = new StoreUUIDHolder(mailstore, UUID);
        mailstoreMap.put(mailstore.toString(), storeUUIDHolder);
      }

      if ((new Date().getTime()) - lastCheckTime > MIN_CHECK_DELTA) {
        for (String mailstoreID : mailstoreMap.keySet()) {
          // Iterate over every mailstore and check for all pending UUIDs
          StoreUUIDHolder currStoreUUIDHolder = mailstoreMap.get(mailstoreID);
          if (currStoreUUIDHolder.UUIDs.size() > 0) {
            logger.log(Level.FINE, "Search for UUID: {0}", UUID);
            boolean connectSuccesful = currStoreUUIDHolder.getMailstore().connect();
            if (!connectSuccesful) {
              throw new MessagingException("Connect to INBOX wasn't successful.");
            }
            for (String currUUID : currStoreUUIDHolder.getAllUUIs()) {
              Message msg = currStoreUUIDHolder.getMailstore().searchForUUID(currUUID);
              if (msg != null) {
                msg.setFlag(Flags.Flag.DELETED, true);
                currStoreUUIDHolder.removeUUID(currUUID);
                foundUUIDs.add(currUUID);
              }
            }
            currStoreUUIDHolder.getMailstore().disconnect(true);
            lastCheckTime = new Date().getTime();
          }
        }
      } else {
        logger.log(Level.FINE, "Don''t search for UUID: {0}. Last check was {1}ms before.",
                new Object[]{UUID, String.valueOf((new Date().getTime()) - lastCheckTime)});
      }

      // Check again in foundUUIDs 
      if (foundUUIDs.remove(UUID)) {
        logger.log(Level.FINE, "Found and remove UUID after search UUID: {0}", UUID);
        return true;
      }
    }

    return false;
  }

  /**
   * Class holds the Mailstore and all UUID which should be searched in the certain store.
   */
  class StoreUUIDHolder {

    private Mailstore mailstore;
    private Set<String> UUIDs;

    private StoreUUIDHolder() {
    }

    public StoreUUIDHolder(Mailstore mailstore, String UUID) {
      this.mailstore = mailstore;
      this.UUIDs = new HashSet<String>();
      this.UUIDs.add(UUID);
    }

    /**
     * Adds a new UUID to the list of UUIDs to be checked.
     *
     * @param UUID
     * @return true if successful
     */
    public boolean addUUID(String UUID) {
      return this.UUIDs.add(UUID);
    }

    /**
     * Removes a UUID from the list of UUIDs to be checked.
     * 
     * @param UUID
     * @return true if successful
     */
    public boolean removeUUID(String UUID) {
      return this.UUIDs.remove(UUID);
    }

    /**
     * Returns UUIDs. 
     * 
     * @return all UUIDs to be checked for the certain Mailstore
     */
    public ArrayList<String> getAllUUIs() {
      ArrayList<String> result = new ArrayList<String>();
      for (String UUID : UUIDs) {
        result.add(UUID);
      }
      return result;
    }

    /**
     * Returns the Mailstore.
     *
     * @return Mailstore
     */
    public Mailstore getMailstore() {
      return this.mailstore;
    }
  }
}
