/*
 * Copyright 2010, cluster:Systems CSG GmbH
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package net.sourceforge.mailprobe.smtp;

import java.io.InputStream;
import java.util.Date;
import net.sourceforge.mailprobe.util.Protocol;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.mail.Message.RecipientType;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import net.sourceforge.mailprobe.util.ExecStats;

/**
 *
 * @author Thomas Tesche <thomas.tesche@clustersystems.de>
 */
public class SMTPSender {

  private Protocol protocol;
  private String hostname;
  private String ehloLocalhost;
  private int port;
  private String username;
  private String password;
  private boolean useStarttls;
  private static final Logger logger = Logger.getLogger(SMTPSender.class.getName());

  /**
   * Do not instantiate empty SMTPSender-class.
   */
  private SMTPSender() {
  }

  /**
   * 
   * @param protocol
   * @param hostname
   * @param ehloLocalhost
   * @param port
   * @param username
   * @param password
   * @param useStarttls
   */
  public SMTPSender(Protocol protocol, String hostname, String ehloLocalhost, String port,
          String username, String password, boolean useStarttls) {
    this.protocol = protocol;
    this.hostname = hostname;
    this.ehloLocalhost = ehloLocalhost;
    if (port != null) {
      this.port = Integer.parseInt(port);
    } else {
      this.port = -1;
    }
    this.username = username;
    this.password = password;
    this.useStarttls = useStarttls;
  }

 /**
  * 
  * @param mimeMessageStream
  * @param uuid
  * @return
  */
  public ExecStats sendMessage(InputStream mimeMessageStream, String uuid) throws Exception {

    Properties props = new Properties(System.getProperties());
    props.put("mail.transport.protocol", protocol.toString());
    //props.put("mail.debug", "true");

    props.setProperty("mail." + protocol.toString() + ".host", hostname);

    if (port > 0) {
      props.setProperty("mail." + protocol.toString() + ".port", Integer.toString(port));
    }
    // 30 seconds timeouts
    props.setProperty("mail." + protocol.toString() + ".connectiontimeout", "30000");
    props.setProperty("mail." + protocol.toString() + ".timeout", "30000");
    if (ehloLocalhost != null) {
      props.setProperty("mail." + protocol.toString() + ".localhost", ehloLocalhost);
    }
    if (username != null && password != null) {
      props.setProperty("mail." + protocol.toString() + ".auth", "true");
    }

    props.setProperty("mail." + protocol.toString() + ".starttls.enable", useStarttls ? "true" : "false");
    props.setProperty("mail." + protocol.toString() + ".ssl.trust", "*");

    // success per default = false, startDate = new Date()
    ExecStats result = new ExecStats(new Date());
//    try {
      Session session = Session.getInstance(props, null);

      MimeMessage msg = new MimeMessage(session, mimeMessageStream);
      msg.setSentDate(new java.util.Date());
      msg.addHeader("X-MailProbe-UUID", uuid);
      msg.saveChanges();

      Transport tr = session.getTransport(protocol.toString());
      if (username != null && password != null) {

        tr.connect(username, password);
      } else {
        tr.connect();
      }

      tr.sendMessage(msg, msg.getAllRecipients());
      result.setSuccess(true);

//    } catch (Exception exception) {
//      logger.log(Level.WARNING, "", exception);
//    }

    result.setEndDate(new Date());
    return result;
  }


  /**
   * 
   * @deprecated
   *
   * @param toAddress
   * @param fromAddress
   * @param uuid
   * @param content
   * @return
   * @throws Exception
   */
  public ExecStats sendStandardMessage(String toAddress, String fromAddress, String uuid,
          String content) {

    Properties props = new Properties(System.getProperties());
    props.put("mail.transport.protocol", protocol.toString());
    //props.put("mail.debug", "true");

    props.setProperty("mail." + protocol.toString() + ".host", hostname);

    if (port > 0) {
      props.setProperty("mail." + protocol.toString() + ".port", Integer.toString(port));
    }
    // 30 seconds timeouts
    props.setProperty("mail." + protocol.toString() + ".connectiontimeout", "30000");
    props.setProperty("mail." + protocol.toString() + ".timeout", "30000");
    if (ehloLocalhost != null) {
      props.setProperty("mail." + protocol.toString() + ".localhost", ehloLocalhost);
    }
    if (username != null && password != null) {
      props.setProperty("mail." + protocol.toString() + ".auth", "true");
    }

    props.setProperty("mail." + protocol.toString() + ".starttls.enable", useStarttls ? "true" : "false");
    props.setProperty("mail." + protocol.toString() + ".ssl.trust", "*");

    // success per default = false, startDate = new Date()
    ExecStats result = new ExecStats(new Date());
    try {
      Session session = Session.getInstance(props, null);
      MimeMessage msg = new MimeMessage(session);
      msg.setFrom(new InternetAddress(fromAddress));
      msg.setRecipient(RecipientType.TO, new InternetAddress(toAddress));

      msg.setSubject("[MAILPROBE] Standard Probe");
      msg.setSentDate(new java.util.Date());
      msg.addHeader("X-MailProbe-UUID", uuid);
      msg.addHeader("X-Mailprobe-Type", "Standard-SMTP");
      msg.setText(content);
      msg.saveChanges();

      Transport tr = session.getTransport(protocol.toString());
      if (username != null && password != null) {

        tr.connect(username, password);
      } else {
        tr.connect();
      }

      tr.sendMessage(msg, msg.getAllRecipients());
      result.setSuccess(true);

    } catch (Exception exception) {
      logger.log(Level.WARNING, "", exception);
    }

    result.setEndDate(new Date());
    return result;
  }
}
