/*
 * Copyright 2010, cluster:Systems CSG GmbH
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package net.sourceforge.mailprobe.store;

import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.search.AndTerm;
import javax.mail.search.HeaderTerm;
import javax.mail.search.SearchTerm;
import javax.mail.search.SubjectTerm;
import net.sourceforge.mailprobe.util.Protocol;

/**
 * Class for accessing mail store. Supported protocols: imap(s), pop3(s).
 * 
 * @author Thomas Tesche <thomas.tesche@clustersystems.de>
 */
public class Mailstore {

  private static final Logger logger = Logger.getLogger(Mailstore.class.getName());
  private Protocol protocol;
  private String hostname;
  private int port;
  private String username;
  private String password;
  private boolean useStarttls;
  private Folder inbox;

  private Mailstore() {
  }

  /**
   *
   * @param protocol imap(s) or pop3(s)
   * @param hostname the server hostname
   * @param port the server port. Could be null for standard ports
   * @param username username for authentication
   * @param password password for authentication
   * @param useStarttls flag whether to use starttls or not
   */
  public Mailstore(Protocol protocol, String hostname, String port,
          String username, String password, boolean useStarttls) {
    this.protocol = protocol;
    this.hostname = hostname;
    if (port != null) {
      this.port = Integer.parseInt(port);
    } else {
      this.port = -1;
    }
    this.username = username;
    this.password = password;
    this.useStarttls = useStarttls;
  }

  @Override
  public String toString() {
    return protocol.toString() + "_" + hostname + "_" + String.valueOf(port)
            + "_" + username + "_" + password + "_" + String.valueOf(useStarttls);
  }

  public Message searchForSubjectAndUUID(String subjectIdent, String uuid) {
    try {
      SearchTerm searchTerm = new AndTerm(
              new SubjectTerm(subjectIdent),
              new HeaderTerm("X-MailProbe-UUID", uuid));

      Message[] msgs = getInbox().search(searchTerm);
      logger.log(Level.FINE, "While searching INBOX: messages found: {0}", msgs.length);
      // Found the message(s). Should be only one but how knows.
      if (msgs.length > 0) {
        return msgs[0];
      } else {
        return null;
      }

    } catch (MessagingException ex) {
      logger.log(Level.WARNING, "While searching INBOX", ex);
      return null;
    }
  }

  public Message searchForUUID(String uuid) {
    try {
      SearchTerm searchTerm = new HeaderTerm("X-MailProbe-UUID", uuid);

      Message[] msgs = getInbox().search(searchTerm);
      logger.log(Level.FINE, "While searching INBOX: messages found: {0}", msgs.length);
      // Found the message(s). Should be only one but how knows.
      if (msgs.length > 0) {
        return msgs[0];
      } else {
        return null;
      }

    } catch (MessagingException ex) {
      logger.log(Level.WARNING, "While searching INBOX", ex);
      return null;
    }
  }

  /**
   * Connects to the INBOX
   *
   * @return true if inbox could opened successful.
   */
  public boolean connect() {
    Folder result = openInbox();
    this.inbox = result;
    return (result != null) ? true : false;
  }

  /**
   * Disconnects from the open folder.
   * @param expunge the folder at disconnect
   * @return true in case of successful disconnect
   */
  public boolean disconnect(boolean expunge) {
    try {
      getInbox().close(expunge);
      return true;

    } catch (MessagingException ex) {
      logger.log(Level.WARNING, "Disconnecting from INBOX", ex);
      return false;
    }
  }

  /**
   * Open the INBOX.
   *
   * @return inbox as folder object
   */
  public Folder openInbox() {

    Properties props = new Properties(System.getProperties());
    //props.put("mail.debug", "true");
    props.setProperty("mail." + protocol.toString() + ".host", hostname);

    if (port > 0) {
      props.setProperty("mail." + protocol.toString() + ".port", Integer.toString(port));
    }
    // 30 seconds timeouts
    props.setProperty("mail." + protocol.toString() + ".connectiontimeout", "30000");
    props.setProperty("mail." + protocol.toString() + ".timeout", "30000");

    props.setProperty("mail." + protocol.toString() + ".starttls.enable", useStarttls ? "true" : "false");
    props.setProperty("mail." + protocol.toString() + ".ssl.trust", "*");

    Session session = Session.getInstance(props, null);
    Folder inboxFolder = null;

    try {
      Store store = session.getStore(protocol.toString());
      if (username != null && password != null) {
        store.connect(username, password);
      } else {
        store.connect();
      }

      inboxFolder = store.getDefaultFolder().getFolder("INBOX");
      inboxFolder.open(Folder.READ_WRITE);

    } catch (MessagingException messagingException) {
      logger.log(Level.WARNING, "", messagingException);

      return null;
    }
    return inboxFolder;
  }

  /**
   * @return the inbox
   */
  public Folder getInbox() {
    return inbox;
  }
}
